/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.reports;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.maven.model.Plugin;
import org.apache.maven.model.ReportPlugin;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import org.sonar.plugins.maven.MavenUtils;
import ch.hortis.sonar.mvn.Report;

/**
 * One ReportHandler per Maven plugin
 */
public abstract class ReportHandler {
  public final void preparePom(MavenProject pom) throws MojoExecutionException {
    Plugin plugin = createPluginWithConfiguration(pom);
    configurePlugin(pom, plugin);
    definePlugin(pom, plugin);
  }

  public abstract void configurePlugin(MavenProject pom, Plugin plugin) throws MojoExecutionException;

  public abstract boolean mustFailOnError();

  public abstract List<String> getGoals();

  private Report report;

  public Report getReport() {
    return report;
  }

  public void setReport(Report report) {
    this.report = report;
  }
  
  public boolean execute( MavenProject pom, Log log ) throws MojoExecutionException {
    return true;
  }

  public boolean executeEvenIfNoJavaSources() {
    return true;
  }

  public boolean executeEvenIfNoJavaTests() {
    return true;
  }

  public final List<String> getCommands() {
    ArrayList<String> goals = new ArrayList<String>();
    String prefix = report.getGroupId() + ":" + report.getArtifactId() + ":" + report.getVersion() + ":";
    for (String goal : getGoals()) {
      goals.add(prefix + goal);
    }
    return goals;
  }

  protected final Plugin createPluginWithConfiguration( MavenProject pom ) {
    Plugin plugin = new Plugin();
    plugin.setGroupId(report.getGroupId());
    plugin.setArtifactId(report.getArtifactId());
    plugin.setVersion(getPluginVersion(pom)); 
    Xpp3Dom configuration = MavenUtils.findPluginConfiguration(pom, report.getGroupId(), report.getArtifactId());
    plugin.setConfiguration(configuration);
    return plugin;
  }
  
  private String getPluginVersion( MavenProject pom ){
    if (!report.isVersionFixed()){    
      Plugin pomPlugin = MavenUtils.findPlugin( pom, report.getGroupId(), report.getArtifactId() );
      if ( pomPlugin != null && pomPlugin.getVersion() != null ){
        return pomPlugin.getVersion();
      }         
    }   
    return report.getVersion();
  }

  protected final void setConfigParameterIfNotAlreadySet(Plugin plugin, String key, String value) {
    String previousValue = MavenUtils.getConfigurationValue(plugin, key);
    if (previousValue == null) {
      setConfigParameter(plugin, key, value);
    }
  }

  protected final void setConfigParameter(Plugin plugin, String key, String value) {
    Xpp3Dom conf = (Xpp3Dom) plugin.getConfiguration();
    if (conf == null) {
      conf = new Xpp3Dom("configuration");
      plugin.setConfiguration(conf);
    }

    Xpp3Dom node = conf.getChild(key);
    if (node == null) {
      node = new Xpp3Dom(key);
      conf.addChild(node);
    }
    node.setValue(value);
  }

  protected final void unsetConfigParameter(Plugin plugin, String key) {
    Xpp3Dom conf = (Xpp3Dom) plugin.getConfiguration();
    if (conf != null) {
      for (int i = 0; i < conf.getChildCount(); i++) {
        Xpp3Dom node = conf.getChild(i);
        if (node.getName().equals(key)) {
          conf.removeChild(i);
          break;
        }
      }
    }
  }

  protected void definePlugin(MavenProject pom, Plugin plugin) {
    if (pom.getBuild().getPlugins() == null) {
      pom.getBuild().setPlugins(new ArrayList<Plugin>());
    } else {
      for (Iterator i = pom.getBuild().getPlugins().iterator(); i.hasNext();) {
        Plugin p = (Plugin) i.next();
        if (p.getArtifactId().equals(plugin.getArtifactId()) && p.getGroupId().equals(p.getGroupId())) {
          i.remove();
        }
      }
    }

    if (pom.getBuild() != null && pom.getBuild().getPluginManagement() != null) {
      for (Iterator i = pom.getBuild().getPluginManagement().getPlugins().iterator(); i.hasNext();) {
        Plugin p = (Plugin) i.next();
        if (p.getArtifactId().equals(plugin.getArtifactId()) && p.getGroupId().equals(p.getGroupId())) {
          i.remove();
        }
      }
    }

    if (pom.getReporting() != null && pom.getReporting().getPlugins() != null) {
      for (Iterator i = pom.getReporting().getPlugins().iterator(); i.hasNext();) {
        ReportPlugin p = (ReportPlugin) i.next();
        if (p.getArtifactId().equals(plugin.getArtifactId()) && p.getGroupId().equals(p.getGroupId())) {
          i.remove();
        }
      }
    }

    pom.getBuild().getPlugins().add(plugin);
  }
}
