/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven.cobertura;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.project.MavenProject;
import org.sonar.commons.metrics.CoreMetrics;
import org.sonar.maven.AbstractCollector;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.java.JavaMeasuresRecorder;
import org.sonar.plugins.maven.MavenUtils;
import org.sonar.plugins.utils.XmlParserException;
import org.sonar.plugins.utils.XmlReportParser;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.File;
import java.text.ParseException;
import java.util.Locale;

public class CoberturaCollector extends AbstractCollector {
  private XmlReportParser parser;
  private JavaMeasuresRecorder recorder;

  public CoberturaCollector(MavenProject project, JavaMeasuresRecorder recorder) {
    File reportFile = findFileFromBuildDirectory(project, "site/cobertura/coverage.xml");
    if (reportFile != null) {
      parser = new XmlReportParser();
      parser.parse(reportFile);
      this.recorder = recorder;
    }
  }

  public CoberturaCollector(File xmlFile, JavaMeasuresRecorder recorder) {
    parser = new XmlReportParser();
    parser.parse(xmlFile);
    this.recorder = recorder;
  }

  public boolean shouldStopOnFailure() {
    return true;
  }

  public boolean shouldExecuteOn(MavenProject mavenProject) {
    return MavenUtils.getLanguage(mavenProject).equals(Languages.JAVA);
  }

  public Metric getMetric(){
    return CoreMetrics.CODE_COVERAGE;
  }

  public void collect() {
    if (parser != null) {
      try {
        collectProjectMeasures();
        collectPackageMeasures();
        collectClassMeasures();

      } catch (ParseException e) {
        throw new XmlParserException(e);
      }
    }
  }

  private void collectProjectMeasures() throws ParseException {
    double lineRate = parseNumber(parser.executeXPath("/coverage/@line-rate"), Locale.ENGLISH);
    recorder.createProjectMeasure(new MeasureKey(getMetric()), convertPercentage(lineRate));
  }

  private void collectPackageMeasures() throws ParseException {
    NodeList packages = parser.executeXPathNodeList("//packages/package");
    for (int i = 0; i < packages.getLength(); i++) {
      Element elt = (Element) packages.item(i);
      recorder.createPackageMeasure(elt.getAttribute("name"), new MeasureKey(getMetric()),
        convertPercentage(parseNumber(elt.getAttribute("line-rate"), Locale.ENGLISH)));
    }
  }

  private void collectClassMeasures() throws ParseException {
    NodeList classes = parser.executeXPathNodeList("//classes/class");
    for (int i = 0; i < classes.getLength(); i++) {
      Element elt = (Element) classes.item(i);
      recordClassMeasure(elt, "line-rate");
    }
  }

  private void recordClassMeasure(Element elt, String valueAttributeName) throws ParseException {
    String name = elt.getAttribute("name");
    String classname = name;
    String packagename = null;
    if (name.indexOf(".") >= 0) {
      classname = StringUtils.substringAfterLast(name, ".");
      packagename = StringUtils.substringBeforeLast(name, ".");
    }
    recorder.createClassMeasure(packagename, classname, false, new MeasureKey(getMetric()), convertPercentage(parseNumber(elt.getAttribute(valueAttributeName), Locale.ENGLISH)));
  }


  private double convertPercentage(Number percentage) {
    return scaleValue(percentage.doubleValue() * 100.0);
  }
}
